#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

class matrix
{
	public:
		matrix(int size = 0);
		~matrix();
		matrix operator + (matrix rhs);
		matrix operator - (matrix rhs);
		matrix operator * (matrix rhs);
		matrix operator = (matrix rhs);
		int set_size(int size);
		int get_size();
		int set_element(double value, int i, int j);
		int get_element(double *dest, int i, int j);
		void fill(double value);
		void fill_random();
	private:
		int m_size;
		double *val;
};

matrix :: matrix(int size)
{
	m_size = size;

	val = (double*)malloc(m_size * m_size * sizeof(double));
	if (val == NULL) {
		printf("\nmatrix creation failed\n");
		while (getchar() != '\n') {};
		getchar();
	}

	if (val == NULL) {
		printf("\nmatrix creation failed\n");
		while (getchar() != '\n') {};
		getchar();
	}

	fill(0);
}

matrix :: ~matrix()
{
	free(val);
}

matrix
matrix :: operator = (matrix rhs)
{
	double tmp;
	int i, j;

	if (m_size != rhs.get_size()) {
		printf("\nsize of matrix must be same\n");
		while (getchar() != '\n') {}
		getchar();
		exit(1);
	}

	for (i = 0; i < m_size; i++) {
		for (j= 0; j < m_size; j++) {
			rhs.get_element(&tmp, i, j);
			val[i * m_size + j] = tmp;
		}
	}
}

matrix
matrix :: operator + (matrix rhs)
{
	matrix *bcp;	
	double tmp;
	int i, j;

	if (m_size != rhs.get_size()) {
		printf("\nsize of matrix must be same\n");
		while (getchar() != '\n') {}
		getchar();
		exit(1);
	}

	for (i = 0; i < m_size; i++) {
		for (j= 0; j < m_size; j++) {
			rhs.get_element(&tmp, i, j);
			val[i * m_size + j] += tmp;
		}
	}

	bcp = new matrix(m_size);
	*bcp = *this;

	for (i = 0; i < m_size; i++) {
		for (j= 0; j < m_size; j++) {
			rhs.get_element(&tmp, i, j);
			val[i * m_size + j] -= tmp;
		}
	}

	return *bcp;
}

matrix
matrix :: operator - (matrix rhs)
{
	matrix *bcp;	
	double tmp;
	int i, j;

	if (m_size != rhs.get_size()) {
		printf("\nsize of matrix must be same\n");
		while (getchar() != '\n') {}
		getchar();
		exit(1);
	}

	for (i = 0; i < m_size; i++) {
		for (j= 0; j < m_size; j++) {
			rhs.get_element(&tmp, i, j);
			val[i * m_size + j] -= tmp;
		}
	}

	bcp = new matrix(m_size);
	*bcp = *this;

	for (i = 0; i < m_size; i++) {
		for (j= 0; j < m_size; j++) {
			rhs.get_element(&tmp, i, j);
			val[i * m_size + j] += tmp;
		}
	}

	return *bcp;
}

matrix
matrix :: operator * (matrix rhs)
{
	matrix *bcp;
	double c, tmp, *t_val;
	int i, j, k;

	if (m_size != rhs.get_size()) {
		printf("\nsize of matrix must be same\n");
		while (getchar() != '\n') {}
		getchar();
		exit(1);
	}

	t_val = (double*)malloc(m_size * m_size * sizeof(*t_val));
	memcpy(t_val, val, m_size * m_size * sizeof(*t_val));

	for (i = 0; i < m_size; i++) {
		for (j = 0; j < m_size; j++) {
			c = 0;
			for (k = 0; k < m_size + 1; k++) {
				rhs.get_element(&tmp,k , j);
				c += t_val[i * m_size + k] * tmp;
			}
			val[i * m_size + j] = c;
		}
	}

	bcp = new matrix(m_size);
	*bcp = *this;
	memcpy(val, t_val, m_size * m_size * sizeof(*t_val));

	return *bcp;

}

int
matrix :: set_size(int size)
{
	double *res;

	res = new double;
	m_size = size;

	res = (double*)realloc((void*)val, m_size * m_size * sizeof(double));
	if (res == NULL) {
		delete res;
		return 0;
	}

	fill(0);
	delete res;

	return 1;
}

int
matrix :: get_size()
{
	return m_size;
}

int
matrix :: set_element(double value, int i, int j)
{
	if (i > m_size) {
		return 0;
	}

	if (j > m_size) {
		return 0;
	}
	val[i * m_size + j] = value;

	return 1;
}

int
matrix :: get_element(double *dest, int i, int j)
{
	if (i > m_size) {
		return 0;
	}

	if (j > m_size) {
		return 0;
	}

	*dest = val[i * m_size + j];

	return 1;
}

void
matrix :: fill(double value)
{
	int i;

	for (i = 0; i < m_size * m_size; i++) {
		val[i] = value;
	}
}

void
matrix :: fill_random()
{
	int i;

	srand((unsigned int)this ^ (unsigned int)val ^ time(NULL));
	for (i = 0; i < m_size * m_size; i++) {
		val[i] = rand() % 10;
	}
}
